#include "detectSign.h"

detectSign::detectSign() {
	lastOperationTime = 0;
	totalTime = 0;
	startTime = 0;
	stopTime = 0;
	operationsCount = 0;
}

double detectSign::getLastOperationTime(){
	lastOperationTime = (stopTime - startTime) / cv::getTickFrequency();
	return lastOperationTime;
}

void detectSign::startTimeMeasure(){
	lastOperationTime = 0;
	totalTime = 0;
	startTime = 0;
	stopTime = 0;
	operationsCount = 0;
}

double detectSign::getTotalTime(){
	return totalTime;
}

double detectSign::getAvgTime(){
	return totalTime / operationsCount;
}

// Opening
void detectSign::opening(cv::Mat in_image,cv::Mat &out_image, int erosion_size, int dilation_size) {
	cv::Mat element;
	cv::Mat temp_image;

 	//Dilatate (Morph Types -> MORPH_RECT,MORPH_ELLIPSE,MORPH_CROSS)
	element = cv::getStructuringElement( cv::MORPH_RECT, cv::Size( 2*dilation_size + 1, 2*dilation_size+1 ),
		cv::Point( dilation_size, dilation_size ) );
  	cv::dilate(in_image, temp_image, element );

	//Erode
	element = cv::getStructuringElement( cv::MORPH_RECT, cv::Size( 2*erosion_size + 1, 2*erosion_size+1 ),
		cv::Point( erosion_size, erosion_size ) );
	cv::erode(temp_image, out_image, element );
}

// Closing
void detectSign::closing(cv::Mat in_image,cv::Mat &out_image, int dilation_size, int erosion_size){
	cv::Mat element;
	cv::Mat temp_image;

	//Erode
	element = cv::getStructuringElement( cv::MORPH_RECT, cv::Size( 2*erosion_size + 1, 2*erosion_size+1 ),
		cv::Point( erosion_size, erosion_size ) );
	cv::erode(in_image, temp_image, element );

 	//Dilatate
	element = cv::getStructuringElement( cv::MORPH_RECT, cv::Size( 2*dilation_size + 1, 2*dilation_size+1 ),
		cv::Point( dilation_size, dilation_size ) );
  	cv::dilate(temp_image, out_image, element );
}

void detectSign::thresholdOtsu(cv::Mat &in_image, cv::Mat &out_image){

    double high_thres;
    double low_thres;

    cv::Mat otsu_thresh;

    high_thres = cv::threshold(in_image, otsu_thresh, 0, 255, CV_THRESH_BINARY+CV_THRESH_OTSU );
    low_thres = 0.5 * high_thres;
    cv::Canny(in_image, out_image, low_thres, high_thres, 3); //3.9

}

void detectSign::redThresh(cv::Mat &image){

	cv::Mat hls_image;
	cv::cvtColor(image, hls_image, CV_BGR2HLS);

	cv::Mat threshed_image_r;
	cv::Mat threshed_image_r2;

	//InRange threshold Red (Red -> Saturation 170-180 + 0-10 na HLS)
	cv::inRange(hls_image, cv::Scalar(160, 20, 40), cv::Scalar(180, 230, 256), threshed_image_r);
	cv::inRange(hls_image, cv::Scalar(0, 20, 40), cv::Scalar(20, 230, 256), threshed_image_r2);
    threshed_image_r +=threshed_image_r2;

	opening(threshed_image_r,threshed_image_r, 1, 1);
	closing(threshed_image_r,threshed_image_r, 1, 1);

    image = threshed_image_r.clone();
}

void detectSign::blueThresh(cv::Mat &image){

	cv::Mat hls_image;
	cv::cvtColor(image, hls_image, CV_BGR2HLS);

	cv::Mat threshed_image_b;

	cv::inRange(hls_image, cv::Scalar(110, 20, 40), cv::Scalar(130, 100, 140), threshed_image_b);

	opening(threshed_image_b,threshed_image_b, 1, 1);
	closing(threshed_image_b,threshed_image_b, 1, 1);

    image = threshed_image_b.clone();
}

bool detectSign::isRed(cv::Mat image){

	redThresh(image);

    int pixelCount = image.rows * image.cols;
    int redPixels = cv::countNonZero(image);

    if (redPixels > (pixelCount / 10)) return true;
    else return false;
}

bool detectSign::isBlue(cv::Mat image){

	blueThresh(image);

    int pixelCount = image.rows * image.cols;
    int redPixels = cv::countNonZero(image);

    if (redPixels > (pixelCount / 8)) return true;
    else return false;
}

bool detectSign::isTriangle(cv::Mat image){

	bool triangle = false;

	if (!isRed(image)) return false;

	redThresh(image);

	cv::Mat canny;
	thresholdOtsu(image, canny);

	std::vector<std::vector<cv::Point> > contours;
	cv::findContours(canny.clone(), contours, CV_RETR_EXTERNAL, CV_CHAIN_APPROX_SIMPLE);

	std::vector<cv::Point> contApprox;
	for (int i = 0; i < contours.size(); i++)
	{
	    cv::approxPolyDP(cv::Mat(contours[i]),contApprox, cv::arcLength(cv::Mat(contours[i]), true) * 0.2, true);
	    if (cv::isContourConvex(contApprox) && cv::contourArea(contApprox) > 100 && contApprox.size() == 3) triangle = true;
	}

	return triangle;
}

bool detectSign::isRectangle(cv::Mat image){

	bool rect = false;

	if (!isBlue(image)) return false;
	else rect = true;

	return rect;
}
