#ifndef DETECTSIGN_H_
#define DETECTSIGN_H_

#include <opencv2/opencv.hpp>
#include <vector>
#include <stdio.h>
#include <time.h>

class detectSign {
protected:
	 double lastOperationTime, totalTime;
	 int64 startTime, stopTime;
	 int operationsCount;

	//Support functions
	void opening(cv::Mat in_image,cv::Mat &out_image, int erosion_size, int dilation_size);
	void closing(cv::Mat in_image,cv::Mat &out_image, int dilation_size, int erosion_size);
	void thresholdOtsu(cv::Mat &in_image, cv::Mat &out_image); // For Threshed images
	void redThresh(cv::Mat &image);
	void blueThresh(cv::Mat &image);

public:
	detectSign();
	virtual ~detectSign() = 0;

	virtual int loadViola(std::string xmlFile) = 0;
	virtual std::vector<cv::Rect> detectViola(cv::Mat image) = 0;

	virtual int loadSVM(std::string datFile) = 0;
	virtual bool detectSVM(cv::Mat image) = 0;
	virtual bool detectHogSVM(cv::Mat image) = 0;

	void 	startTimeMeasure();
	double 	getLastOperationTime();
	double 	getTotalTime();
	double	getAvgTime();

	bool isTriangle(cv::Mat image);
	bool isRectangle(cv::Mat image);
	bool isRed(cv::Mat image);
	bool isBlue(cv::Mat image);

};

#endif /* DETECTSIGN_H_ */



