#include "detectSignCpu.h"


int detectSignCpu::loadViola(std::string xmlFile){

	startTimeMeasure();

	cascade.load(xmlFile);

	if (cascade.empty()) {
		std::cout<<"Could not open Cascade Model"<<std::endl;
		return 1;
	}
	else return 0;
}

std::vector<cv::Rect> detectSignCpu::detectViola(cv::Mat image) {

     std::vector<cv::Rect> objects;

     startTime = cv::getTickCount();
     cascade.detectMultiScale(image, objects, 1.1, 4, 0 , cv::Size(24,24), cv::Size(400,400) ); //
     stopTime = cv::getTickCount();

     totalTime += getLastOperationTime();
     operationsCount++;

     return objects;
}

void detectSignCpu::getHog(const cv::Mat rect, std::vector<float> &feature){
	cv::Mat image;
	cv::cvtColor(rect, image, CV_BGR2GRAY);

	//Error checks
	if (image.empty()){
		feature.clear();
		return;
	}

	resize(image,image, cv::Size(64,64));
    std::vector<cv::Point> locations;

    startTime = cv::getTickCount();
    hog.compute(image, feature, cv::Size(8, 8), cv::Size(0, 0), locations);
    stopTime = cv::getTickCount();

    totalTime += getLastOperationTime();

    image.release();
}

int detectSignCpu::loadSVM(std::string datFile){
	hog.winSize = cv::Size(64,64);
	svm.load(datFile.c_str());

	svmHog = cv::HOGDescriptor(cv::Size(64,64), cv::Size(16, 16), cv::Size(8, 8), cv::Size(8, 8), 9);

	std::vector<float> supportVector;
	svm.getSupportVector(supportVector);
	svmHog.setSVMDetector(supportVector);

	return 0;
}

bool detectSignCpu::detectSVM(cv::Mat image){

	int hogFeaturesCount = 1764;  // (Hog Feature Size)

	cv::Mat data(1,hogFeaturesCount,CV_32FC1);
	std::vector<float> hogFeature;

	getHog(image, hogFeature);

	for (int i=0; i < hogFeaturesCount; i++) {
		data.at<float>(0,i) = hogFeature.at(i);
	}

	float result = svm.predict(data);

	if (result > 0) return true;
	else return false;
}

bool detectSignCpu::detectHogSVM(cv::Mat image){

 	std::vector<cv::Point> objHog;

 	if (image.rows < svmHog.winSize.height || image.cols < svmHog.winSize.width)
 	resize(image,image, cv::Size(svmHog.winSize.height,svmHog.winSize.width));

 	svmHog.detect(image,objHog);

 	if (objHog.size() > 0) return true;
 	else return false;
}

