#include "detectSignGpu.h"


int detectSignGpu::loadViola(std::string xmlFile){

	startTimeMeasure();

	cascade.load(xmlFile);

	if (cascade.empty()) {
		std::cout<<"Could not open Cascade Model"<<std::endl;
		return 1;
	}
	else return 0;
}

std::vector<cv::Rect> detectSignGpu::detectViola(cv::Mat image) {

    cv::gpu::GpuMat gpuImage(image);
    cv::gpu::GpuMat gpuObjects;
    cv::Mat 		gpuObjectsDownload;
    std::vector<cv::Rect> objects;

    startTime = cv::getTickCount();
    int objectCount = cascade.detectMultiScale(gpuImage, gpuObjects, cv::Size(400,400), cv::Size(24,24), 1.1, 4 );
    stopTime = cv::getTickCount();

    if (objectCount > 0) {

    	gpuObjects.colRange(0, objectCount).download(gpuObjectsDownload);
    	cv::Rect* hostObjects = gpuObjectsDownload.ptr<cv::Rect>();

    	for(int i = 0; i < objectCount; ++i)
    		objects.push_back(hostObjects[i]);
    }

    totalTime += getLastOperationTime();
    operationsCount++;

    return objects;
}

void detectSignGpu::getHog(const cv::Mat rect, std::vector<float> &feature){
	cv::Mat image;
	cv::cvtColor(rect, image, CV_BGR2GRAY);
	resize(image,image, cv::Size(64,64));

	cv::gpu::GpuMat gpuImage;
    cv::gpu::GpuMat gpuDescriptor;

	gpuImage.upload(image);

    startTime = cv::getTickCount();
    hog.getDescriptors(gpuImage, cv::Size(8, 8), gpuDescriptor);
    stopTime = cv::getTickCount();

    cv::Mat cpuDescriptor = cv::Mat(gpuDescriptor);
    cpuDescriptor.copyTo(feature);

    totalTime += getLastOperationTime();

    image.release();
}

int detectSignGpu::loadSVM(std::string datFile){
	hog.win_size = cv::Size(64,64);

	svm.load(datFile.c_str());

	svmHog = cv::gpu::HOGDescriptor(cv::Size(64,64), cv::Size(16, 16), cv::Size(8, 8), cv::Size(8, 8), 9);
	std::vector<float> supportVector;
	svm.getSupportVector(supportVector);
	svmHog.setSVMDetector(supportVector);

	return 0;

}

bool detectSignGpu::detectSVM(cv::Mat image){

	int hogFeaturesCount = 1764;  // (Hog Feature Size)

	cv::Mat data(1,hogFeaturesCount,CV_32FC1);
	std::vector<float> hogFeature;

	getHog(image, hogFeature);

	for (int i=0; i < hogFeaturesCount; i++) {
		data.at<float>(0,i) = hogFeature.at(i);
	}

	float result = svm.predict(data);

	if (result > 0) return true;
	else return false;

}

bool detectSignGpu::detectHogSVM(cv::Mat image){

	cv::cvtColor(image , image ,  CV_BGR2GRAY);

 	if (image.rows < svmHog.win_size.height || image.cols < svmHog.win_size.width)
 	resize(image,image, cv::Size(svmHog.win_size.height,svmHog.win_size.width));

	cv::gpu::GpuMat gpuImage;
	gpuImage.upload(image);

 	std::vector<cv::Point> objHog;

 	startTime = cv::getTickCount();
 	svmHog.detect(gpuImage,objHog);
 	stopTime = cv::getTickCount();

 	if (objHog.size() > 0) return true;
 	else return false;


    totalTime += getLastOperationTime();

    gpuImage.release();

}

