/*
 * Autor: Karel Zídek
 * Soubor: gui.cpp
 *
 * Road Sign Detector GUI
 */

#include "gui.h"

RsdGui::RsdGui(){
	//Create Basic Images
	m_canvas.create(600,1000, CV_8UC3);
	m_canvas  = cv::Scalar(242, 241, 240);

	m_mainImage = cv::Mat::zeros(10,10, CV_8UC3);

	m_sign0Image = cv::Mat::zeros(10,10, CV_8UC3);
	m_sign1Image = cv::Mat::zeros(10,10, CV_8UC3);
	m_sign2Image = cv::Mat::zeros(10,10, CV_8UC3);
	m_sign3Image = cv::Mat::zeros(10,10, CV_8UC3);

	m_sign0ImageG = cv::Mat::zeros(10,10, CV_8UC3);
	m_sign1ImageG = cv::Mat::zeros(10,10, CV_8UC3);
	m_sign2ImageG = cv::Mat::zeros(10,10, CV_8UC3);
	m_sign3ImageG = cv::Mat::zeros(10,10, CV_8UC3);

	performanceTextC = "";
	performanceTextG = "";
}

RsdGui::~RsdGui(){		
}

void RsdGui::setInfoTextC(std::string text){
	performanceTextC = text;
}

void RsdGui::setInfoTextG(std::string text){
	performanceTextG = text;
}

void RsdGui::drawCanvas(){
	//cvStartWindowThread();
	cv::namedWindow( "RSD Image", CV_WINDOW_AUTOSIZE | CV_WINDOW_KEEPRATIO | CV_GUI_NORMAL ); // CV_GUI_EXPANDED 
	cv::moveWindow("RSD Image", 40, 40);
	
	//Clean Canvas
	m_canvas  = cv::Scalar(242, 241, 240);

	//Draw Main Image
	cv::Mat roi(m_canvas(cv::Rect(20,20,600,500)));

	cv::Size main_size(600,500);
	cv::resize(m_mainImage, m_mainImage, main_size);

	m_mainImage.copyTo(roi);

	//Draw Main Image
	cv::Mat roi0(m_canvas(cv::Rect(640,20,100,100)));
	cv::Mat roi1(m_canvas(cv::Rect(640,155,100,100)));
	cv::Mat roi2(m_canvas(cv::Rect(640,285,100,100)));
	cv::Mat roi3(m_canvas(cv::Rect(640,420,100,100)));

	m_sign0Image.copyTo(roi0);
	m_sign1Image.copyTo(roi1);
	m_sign2Image.copyTo(roi2);
	m_sign3Image.copyTo(roi3);

	//Draw Main Image
	cv::Mat roi0G(m_canvas(cv::Rect(760,20,100,100)));
	cv::Mat roi1G(m_canvas(cv::Rect(760,155,100,100)));
	cv::Mat roi2G(m_canvas(cv::Rect(760,285,100,100)));
	cv::Mat roi3G(m_canvas(cv::Rect(760,420,100,100)));

	m_sign0ImageG.copyTo(roi0G);
	m_sign1ImageG.copyTo(roi1G);
	m_sign2ImageG.copyTo(roi2G);
	m_sign3ImageG.copyTo(roi3G);

	cv::putText( m_canvas, "CPU", cv::Point(680,15),cv::FONT_HERSHEY_SIMPLEX, 0.5, cv::Scalar(255,0,0),1,8,false );
	cv::putText( m_canvas, "GPU", cv::Point(800,15),cv::FONT_HERSHEY_SIMPLEX, 0.5, cv::Scalar(0,0,255),1,8,false );

	cv::putText( m_canvas, performanceTextC, cv::Point(20,550),cv::FONT_HERSHEY_SIMPLEX, 0.5, cv::Scalar(0,0,0),1,8,false );
	cv::putText( m_canvas, performanceTextG, cv::Point(20,570),cv::FONT_HERSHEY_SIMPLEX, 0.5, cv::Scalar(0,0,0),1,8,false );

	//Show
	cv::imshow("RSD Image", m_canvas);

	cv::waitKey(1);
}

void RsdGui::setMainImage(cv::Mat mainImage){
	m_mainImage = mainImage.clone();
}

void RsdGui::setSignImageC(cv::Mat signImage){
	m_sign3Image = m_sign2Image;
	m_sign2Image = m_sign1Image;
	m_sign1Image = m_sign0Image;

	cv::Size im_size(100,100);
	m_sign0Image = signImage.clone();
	cv::resize(m_sign0Image, m_sign0Image, im_size);
}

void RsdGui::setSignImageG(cv::Mat signImage){
	m_sign3ImageG = m_sign2ImageG;
	m_sign2ImageG = m_sign1ImageG;
	m_sign1ImageG = m_sign0ImageG;

	cv::Size im_size(100,100);
	m_sign0ImageG = signImage.clone();
	cv::resize(m_sign0ImageG, m_sign0ImageG, im_size);
}

void RsdGui::cleanSmall(){
	m_sign0Image.release();
	m_sign1Image.release();
	m_sign2Image.release();
	m_sign3Image.release();

	m_sign0ImageG.release();
	m_sign1ImageG.release();
	m_sign2ImageG.release();
	m_sign3ImageG.release();
}
