#include <opencv2/opencv.hpp>

#include <iostream>
#include <dirent.h>
#include <stdio.h>
#include <algorithm>
#include <string>
#include <vector>

#include "gui.h"
#include "detectSign.h"
#include "detectSignCpu.h"
#include "detectSignGpu.h"

//Choose Sign Type -- RED or BLUE
#define RED
//#define BLUE

//Path To Images
const std::string  PATH = "./data/geodis/CAM2/0/";

#ifdef RED

//Red triangle Data
const std::string  XML = "./xml/VJ_cascade_Type1.xml";
const std::string  SVM = "./xml/SVM_data_Type1.dat";

#endif
#ifdef BLUE

//Blue Rectangle Data
const std::string  XML = "./xml/VJ_cascade_Type2.xml";
const std::string  SVM = "./xml/SVM_data_Type2.dat";

#endif

bool isJpeg(std::string file){
	std::string ext  = file.substr(file.find_last_of(".") + 1);
	std::transform(ext.begin(), ext.end(), ext.begin(), ::tolower);

	if ( ext == "jpg") return true;
	else return false;
}

std::vector<std::string> getFileList(){
	DIR*     dir;
	dirent*  files;
	std::vector <std::string> filelist;

	//Read Directory Files
	dir = opendir(PATH.c_str());

	if(dir == NULL){
		std::cout<<"Directory Error"<<std::endl;
	}

	while (files = readdir(dir)){
	      filelist.push_back( std::string(files->d_name) );
	}
    closedir(dir);

    std::sort(filelist.begin(), filelist.end());

    return filelist;
}

int main(int argc, char* argv[]){

	//Get Folder Listings
	std::vector <std::string> filelist;
	filelist = getFileList();

    //Prepare Canvas
    RsdGui* gui = new RsdGui();

    //Create Detector Class CPU
    detectSign *detectCpu = new detectSignCpu();
    detectCpu->loadViola(XML);
    detectCpu->loadSVM(SVM);

    //Create Detector Class GPU
    detectSign *detectGpu = new detectSignGpu();
    detectGpu->loadViola(XML);
    detectGpu->loadSVM(SVM);

    //Application Variables
    int imgNumber = 0;
    int hitCountCPU=0,hitCountGPU=0,totalCPU=0,totalGPU=0,signCPU=0,signGPU=0;
    bool pause = false;

    //For all files
    for (std::vector<std::string>::iterator it = filelist.begin() ; it != filelist.end(); ++it) {

		std::string fileName = *it;
        std::string path = PATH + fileName;

		if (isJpeg(fileName)) {

			 imgNumber++;
	         std::cout << "Processing Image "<<imgNumber<<": "<<fileName<< std::endl;

	         gui->cleanSmall();

	         cv::Mat original = cv::imread(path.c_str());

	         //Preprocess input Image
	         if( !original.empty() ) resize(original, original, cv::Size(original.cols/2, original.rows/2));
	         cv::Mat work_img = original.clone();
	         cv::cvtColor(original, work_img, cv::COLOR_RGB2GRAY);
	         cv::equalizeHist(work_img, work_img);

	         //Viola - CPU
	         std::vector<cv::Rect> objectsCpu;
	         objectsCpu = detectCpu->detectViola(work_img);

	         //Viola - GPU
	         std::vector<cv::Rect> objectsGpu;
	         objectsGpu = detectGpu->detectViola(work_img);

	         //Draw and display
	         cv::Mat origRects =original.clone();
	         gui->setMainImage(origRects);
	         gui->setInfoTextC("CPU: Operation Time: "+std::to_string(detectCpu->getLastOperationTime())+
	        		 "s | Total time: "+ std::to_string(detectCpu->getTotalTime())+"s | Avg Time: "+
	        		 std::to_string(detectCpu->getAvgTime())+"s | Viola:"+
	        		 std::to_string(totalCPU)+" SVM:"+std::to_string(hitCountCPU)+" Signs:"+std::to_string(signCPU));

	         gui->setInfoTextG("GPU: Operation Time: "+std::to_string(detectGpu->getLastOperationTime())+
	        		 "s | Total time: "+ std::to_string(detectGpu->getTotalTime())+"s | Avg Time: "+
	        		 std::to_string(detectGpu->getAvgTime())+"s | Viola:"+
	        		 std::to_string(totalGPU)+" SVM:"+std::to_string(hitCountGPU)+" Signs:"+std::to_string(signGPU));

	         totalCPU+= (int)objectsCpu.size();
	         totalGPU+= (int)objectsGpu.size();

	         //CPU Objects
	         for(int i=0; i < (int)objectsCpu.size(); i++){

	        	 if (detectCpu->detectSVM(original(objectsCpu[i]).clone())){

	        		 hitCountCPU++;

	        		 //Draw Rects
	        		 rectangle(origRects, objectsCpu[i], cv::Scalar(255,0,0), 2);
#ifdef BLUE
	        		if (detectCpu->isRectangle(original(objectsCpu[i]).clone()) ) {
#endif
#ifdef RED
			        if (detectCpu->isTriangle(original(objectsCpu[i]).clone()) ) {
#endif
			        	signCPU++;
			        	pause = true;

			        	cv::Mat object = original(objectsCpu[i]);
				        gui->setSignImageC(object);
					    gui->drawCanvas();
			        }
		         }
	         }

	         //GPU Objects
	         for(int i=0; i < (int)objectsGpu.size(); i++){

	        	 if (detectGpu->detectHogSVM(original(objectsGpu[i]).clone())){

	        		 hitCountGPU++;

		        	 //Draw Rects
		        	 rectangle(origRects, objectsGpu[i], cv::Scalar(0,0,255), 2);
#ifdef BLUE
		        	    if (detectGpu->isRectangle(original(objectsGpu[i]).clone()) ) {
#endif
#ifdef RED
				        if (detectGpu->isTriangle(original(objectsGpu[i]).clone()) ) {
#endif
				        	signGPU++;
				        	pause = true;

				        	cv::Mat object = original(objectsGpu[i]);
					        gui->setSignImageG(object);
						    gui->drawCanvas();
				        }
		         }
	         }


	         objectsCpu.clear();
	         objectsGpu.clear();

	         gui->setMainImage(origRects);
	         gui->drawCanvas();

	         if (pause) {
	        	 cv::waitKey();
	        	 pause = false;
	         }
		} // isJpeg
    } //Files Loop

    gui->setInfoTextC("Total time: "+ std::to_string(detectCpu->getTotalTime())+"s | Avg Time: "+
   		 std::to_string(detectCpu->getAvgTime())+"s | Viola:"+
   		 std::to_string(totalCPU)+" SVM:"+std::to_string(hitCountCPU)+" Signs:"+std::to_string(signCPU));

    gui->setInfoTextG("Total time: "+ std::to_string(detectGpu->getTotalTime())+"s | Avg Time: "+
   		 std::to_string(detectGpu->getAvgTime())+"s | Viola:"+
   		 std::to_string(totalGPU)+" SVM:"+std::to_string(hitCountGPU)+" Signs:"+std::to_string(signGPU));

    std::cout<<"	Total detection time: "<<detectCpu->getTotalTime()<<" s"<<std::endl;
    cv::waitKey();
    return 0;
}
