#!/bin/bash

echo
echo "**************************************************"
echo "               OpenCV HAAR Generating             "
echo "**************************************************"
echo
echo "**************************************************"
echo "                    Cleaning                      "
echo "**************************************************" 
echo

rm ./classifier/*
rm positive.vec
rm positive.txt
rm positives.txt
rm negative.txt
rm positive_generated/0/*
rm positive_generated/1/*
rm positive_generated/2/*
rm positive_generated/3/*
rm positive_generated/4/*
rm positive_generated/5/*

echo
echo "**************************************************"
echo "              Detecting negatives...               "
echo "**************************************************"
echo

find ./negative -name *.jpg > negative.txt

echo
echo "**************************************************"
echo "          Generating Positive Samples...          "
echo "**************************************************"
echo

opencv_createsamples -img positive/01.png -bg negative.txt -bgcolor 0 -bgthresh 0 -num 200 -maxxangle 0.2 -maxyangle 0.3 -maxzangle 0.4  -w 48 -h 48 -info positive_generated/0/annotations.lst
opencv_createsamples -img positive/02.png -bg negative.txt -bgcolor 0 -bgthresh 0 -num 200 -maxxangle 0.2 -maxyangle 0.3 -maxzangle 0.4  -w 48 -h 48 -info positive_generated/1/annotations.lst
opencv_createsamples -img positive/03.png -bg negative.txt -bgcolor 0 -bgthresh 0 -num 200 -maxxangle 0.2 -maxyangle 0.3 -maxzangle 0.4  -w 48 -h 48 -info positive_generated/2/annotations.lst
opencv_createsamples -img positive/04.png -bg negative.txt -bgcolor 0 -bgthresh 0 -num 200 -maxxangle 0.2 -maxyangle 0.3 -maxzangle 0.4  -w 48 -h 48 -info positive_generated/3/annotations.lst
opencv_createsamples -img positive/05.png -bg negative.txt -bgcolor 0 -bgthresh 0 -num 200 -maxxangle 0.2 -maxyangle 0.3 -maxzangle 0.4  -w 48 -h 48 -info positive_generated/4/annotations.lst
opencv_createsamples -img positive/06.png -bg negative.txt -bgcolor 0 -bgthresh 0 -num 200 -maxxangle 0.2 -maxyangle 0.3 -maxzangle 0.4  -w 48 -h 48 -info positive_generated/5/annotations.lst

echo
echo "**************************************************"
echo "            Creating Positive file...             "
echo "**************************************************"
echo

sed 's/^/.\/positive_generated\/0\//' positive_generated/0/annotations.lst  >> positive.txt
sed 's/^/.\/positive_generated\/1\//' positive_generated/1/annotations.lst  >> positive.txt
sed 's/^/.\/positive_generated\/2\//' positive_generated/2/annotations.lst  >> positive.txt
sed 's/^/.\/positive_generated\/3\//' positive_generated/3/annotations.lst  >> positive.txt
sed 's/^/.\/positive_generated\/4\//' positive_generated/4/annotations.lst  >> positive.txt
sed 's/^/.\/positive_generated\/5\//' positive_generated/5/annotations.lst  >> positive.txt

echo
echo "**************************************************"
echo "        Shuffle Items in Positive File            "
echo "**************************************************" 
echo

# Shuffle code by ghostdog74: http://stackoverflow.com/questions/2153882/how-can-i-shuffle-the-lines-of-a-text-file-in-unix-command-line
awk 'BEGIN{srand() }
{ lines[++d]=$0 }
END{
    while (1){
    if (e==d) {break}
        RANDOM = int(1 + rand() * d)
        if ( RANDOM in lines  ){
            print lines[RANDOM]
            delete lines[RANDOM]
            ++e
        }
    }
}' positive.txt > positives.txt

echo
echo "**************************************************"
echo "                Creating Samples...               "
echo "**************************************************"
echo

opencv_createsamples -info positives.txt -bg negative.txt -bgcolor 0 -bgthresh 0 -num 1200 -maxxangle 0.2 -maxyangle 0.3 -maxzangle 0.4 -w 48 -h 48 -vec positive.vec

echo
echo "**************************************************"
echo "                Generating Done                   "
echo "**************************************************"
echo 
